/*! \file can.h \brief CAN driver with limited can support. */
// *****************************************************************************
//
// File Name	: 'can.h'
// Title		: CAN driver with limited can support
// Author		: Markus Schiesser 2007
// Created		: 01/01/2007
// Revised		: 04/27/2007
// Version		: 0.7
// Target MCU	: ATMEL AVR Series
// Editor Tabs	: 3
//
// Supports:	extended frame ID (CAN 2.0B)
//				interrupt driven RX/TX
//				polling RX/TX
//				up to 15 message objects
// Not supported:
//				standart frame ID (CAN2.0A)
//				remote frames
//				Frame buffer mode
//				and few other things		
//
// *****************************************************************************

#ifndef _CAN_H_
#define _CAN_H_




//------ I N C L U D E S -----

#include "library/can_config.h"

//extern struct MOb_struct MOb_TX;
//extern struct MOb_struct MOb_RX;
//extern struct CAN_Service_struct CAN_Service;

//----- D E F I N I T I O N S ----- 

#ifndef F_CPU
#  error  You must define F_CPU in "config.h" file
#endif
    
#ifndef CAN_BAUDRATE
#  error  You must define CAN_BAUDRATE in "can_config.h" file
#endif

   
#if F_CPU == 16000             //!< Fclkio = 16 MHz, Tclkio = 62.5 ns
#   if   CAN_BAUDRATE == 100       //!< -- 100Kb/s, 16x Tscl, sampling at 75%
#       define CONF_CANBT1  0x12       // Tscl  = 10x Tclkio = 625 ns
#       define CONF_CANBT2  0x0C       // Tsync = 1x Tscl, Tprs = 7x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x37       // Tpsh1 = 4x Tscl, Tpsh2 = 4x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 125       //!< -- 125Kb/s, 16x Tscl, sampling at 75%
#       define CONF_CANBT1  0x0E       // Tscl  = 8x Tclkio = 500 ns
#       define CONF_CANBT2  0x0C       // Tsync = 1x Tscl, Tprs = 7x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x37       // Tpsh1 = 4x Tscl, Tpsh2 = 4x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 200       //!< -- 200Kb/s, 16x Tscl, sampling at 75%
#       define CONF_CANBT1  0x08       // Tscl  = 5x Tclkio = 312.5 ns
#       define CONF_CANBT2  0x0C       // Tsync = 1x Tscl, Tprs = 7x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x37       // Tpsh1 = 4x Tscl, Tpsh2 = 4x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 250       //!< -- 250Kb/s, 16x Tscl, sampling at 75%
#       define CONF_CANBT1  0x06       // Tscl  = 4x Tclkio = 250 ns
#       define CONF_CANBT2  0x0C       // Tsync = 1x Tscl, Tprs = 7x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x37       // Tpsh1 = 4x Tscl, Tpsh2 = 4x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 500       //!< -- 500Kb/s, 8x Tscl, sampling at 75%
#       define CONF_CANBT1  0x06       // Tscl = 4x Tclkio = 250 ns
#       define CONF_CANBT2  0x04       // Tsync = 1x Tscl, Tprs = 3x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x13       // Tpsh1 = 2x Tscl, Tpsh2 = 2x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 1000      //!< -- 1 Mb/s, 8x Tscl, sampling at 75%
#       define CONF_CANBT1  0x02       // Tscl  = 2x Tclkio = 125 ns
#       define CONF_CANBT2  0x04       // Tsync = 1x Tscl, Tprs = 3x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x13       // Tpsh1 = 2x Tscl, Tpsh2 = 2x Tscl, 3 sample points
#   else
#       error This CAN_BAUDRATE value is not in "can.h" file
#   endif

#elif F_CPU == 12000           //!< Fclkio = 12 MHz, Tclkio = 83.333 ns
#   if   CAN_BAUDRATE == 100       //!< -- 100Kb/s, 20x Tscl, sampling at 75%
#       define CONF_CANBT1  0x0A       // Tscl  = 6x Tclkio = 500 ns
#       define CONF_CANBT2  0x0E       // Tsync = 1x Tscl, Tprs = 8x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x4B       // Tpsh1 = 6x Tscl, Tpsh2 = 5x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 125       //!< -- 125Kb/s, 16x Tscl, sampling at 75%
#       define CONF_CANBT1  0x0A       // Tscl  = 6x Tclkio = 500 ns
#       define CONF_CANBT2  0x0C       // Tsync = 1x Tscl, Tprs = 7x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x37       // Tpsh1 = 4x Tscl, Tpsh2 = 4x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 200       //!< -- 200Kb/s, 20x Tscl, sampling at 75%
#       define CONF_CANBT1  0x04       // Tscl  = 3x Tclkio = 250 ns
#       define CONF_CANBT2  0x0E       // Tsync = 1x Tscl, Tprs = 8x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x4B       // Tpsh1 = 6x Tscl, Tpsh2 = 5x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 250       //!< -- 250Kb/s, 16x Tscl, sampling at 75%
#       define CONF_CANBT1  0x04       // Tscl  = 3x Tclkio = 250 ns
#       define CONF_CANBT2  0x0C       // Tsync = 1x Tscl, Tprs = 7x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x37       // Tpsh1 = 4x Tscl, Tpsh2 = 4x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 500       //!< -- 500Kb/s, 12x Tscl, sampling at 75%
#       define CONF_CANBT1  0x02       // Tscl  = 2x Tclkio = 166.666 ns
#       define CONF_CANBT2  0x08       // Tsync = 1x Tscl, Tprs = 5x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x25       // Tpsh1 = 3x Tscl, Tpsh2 = 3x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 1000      //!< -- 1 Mb/s, 12x Tscl, sampling at 75%
#       define CONF_CANBT1  0x00       // Tscl  = 1x Tclkio = 83.333 ns
#       define CONF_CANBT2  0x08       // Tsync = 1x Tscl, Tprs = 5x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x25       // Tpsh1 = 3x Tscl, Tpsh2 = 3x Tscl, 3 sample points
#   else
#       error This CAN_BAUDRATE value is not in "can.h" file
#   endif

#elif F_CPU == 8000              //!< Fclkio = 8 MHz, Tclkio = 125 ns
#   if   CAN_BAUDRATE == 100       //!< -- 100Kb/s, 16x Tscl, sampling at 75%
#       define CONF_CANBT1  0x08       // Tscl  = 5x Tclkio = 625 ns
#       define CONF_CANBT2  0x0C       // Tsync = 1x Tscl, Tprs = 7x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x37       // Tpsh1 = 4x Tscl, Tpsh2 = 4x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 125       //!< -- 125Kb/s, 16x Tscl, sampling at 75%
#       define CONF_CANBT1  0x06       // Tscl  = 4x Tclkio = 500 ns
#       define CONF_CANBT2  0x0C       // Tsync = 1x Tscl, Tprs = 7x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x37       // Tpsh1 = 4x Tscl, Tpsh2 = 4x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 200       //!< -- 200Kb/s, 20x Tscl, sampling at 75%
#       define CONF_CANBT1  0x02       // Tscl  = 2x Tclkio = 250 ns
#       define CONF_CANBT2  0x0E       // Tsync = 1x Tscl, Tprs = 8x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x4B       // Tpsh1 = 6x Tscl, Tpsh2 = 5x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 250       //!< -- 250Kb/s, 16x Tscl, sampling at 75%
#       define CONF_CANBT1  0x02       // Tscl  = 2x Tclkio = 250 ns
#       define CONF_CANBT2  0x0C       // Tsync = 1x Tscl, Tprs = 7x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x37       // Tpsh1 = 4x Tscl, Tpsh2 = 4x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 500       //!< -- 500Kb/s, 8x Tscl, sampling at 75%
#       define CONF_CANBT1  0x02       // Tscl  = 2x Tclkio = 250 ns
#       define CONF_CANBT2  0x04       // Tsync = 1x Tscl, Tprs = 3x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x13       // Tpsh1 = 2x Tscl, Tpsh2 = 2x Tscl, 3 sample points
#   elif CAN_BAUDRATE == 1000      //!< -- 1 Mb/s, 8x Tscl, sampling at 75%
#       define CONF_CANBT1  0x00       // Tscl  = 1x Tclkio = 125 ns
#       define CONF_CANBT2  0x04       // Tsync = 1x Tscl, Tprs = 3x Tscl, Tsjw = 1x Tscl
#       define CONF_CANBT3  0x13       // Tpsh1 = 2x Tscl, Tpsh2 = 2x Tscl, 3 sample points
#   else
#       error This CAN_BAUDRATE value is not in "can_config.h" file
#   endif
#else
#   error This F_CPU value is not in "config.h" file
#endif




// ----- M A R K R O S -----

//CAN Enable
#define CAN_ENABLE			      {CANGCON |= (1<<ENASTB);}
//CAN Disable
#define CAN_STANDBY				  {CANGCON &= (~(1<<ENASTB);}
//CAN Reset
#define CAN_RESET			      {CANGCON = (1<<SWRES);}

/* Configure CAN Interrupte Sources */  

//CAN General Interrupt
#define CAN_IRQ_ENABLE            { CANGIE |= (1<<ENIT); }		//Enable
#define CAN_IRQ_DISABLE           { CANGIE &= (~(1<<ENIT)); }   //Disable
//CAN Bus Off Interrupt
#define CAN_ENBOFF_IRQ_ENABLE     { CANGIE |= (1<<ENBOFF); }	//Enable
#define CAN_ENBOFF_IRQ_DISABLE    { CANGIE &= (~(1<<ENBOFF));}  //Disable
//CAN Receive Interrupt
#define CAN_RX_IRQ_ENABLE         { CANGIE |= (1<<ENRX);  }		//Enable
#define CAN_RX_IRQ_DISABLE        { CANGIE &= (~(1<<ENRX)); }   //Disable
//CAN Transmitt Interrupt
#define CAN_TX_IRQ_ENABLE         { CANGIE |= (1<<ENTX);  }		//Enable
#define CAN_TX_IRQ_DISABLE        { CANGIE &= (~(1<<ENTX)); }   //Disable
//CAN MOb Error Interrupt
#define CAN_ENERR_IRQ_ENABLE      { CANGIE |= (1<<ENERR);  }	//Enable
#define CAN_ENERR_IRQ_DISABLE     { CANGIE &= (~(1<<ENERR)); }  //Disable
//CAN Frame buffer Interrupt
#define CAN_ENBX_IRQ_ENABLE       { CANGIE |= (1<<ENBX);  }		//Enable
#define CAN_ENBX_IRQ_DISABLE      { CANGIE &= (~(1<<ENBX)); }	//Disable
//CAN General Error Interrupt
#define CAN_ENERG_IRQ_ENABLE      { CANGIE |= (1<<ENERG);  }	//Enable
#define CAN_ENERG_IRQ_DISABLE     { CANGIE &= (~(1<<ENERG)); }	//Disable
//CAN Timer Overrun Interrupt
#define CAN_ENOVRT_IRQ_ENABLE     { CANGIE |= (1<<ENOVRT); }	//Enable
#define CAN_ENOVRT_IRQ_DISABLE    { CANGIE &= (~(1<<ENOVRT)); } //Disable

//Enable / Disable interrupt for the mob
#define CAN_IRQ_ENABLE_MOB(mob)	  {(((mob) < (8))?(CANIE2 |= (1<<(mob))):(CANIE1 |= (1<<mob - 8)));}
#define CAN_IRQ_DISABLE_MOB(mob)  {(((mob) < (8))?(CANIE2 &= (~(1<<(mob)))):(CANIE1 &= (~(1<<mob - 8))));}

//Clear Interruptes
#define CAN_GENERAL_IRQ_CLEAR     {CANGIT &= (~(0x7F));}
#define CAN_ERROR_IRQ_CLEAR		  {CANSTMOB &= (~(0x1F));}
#define CAN_TXOK_IRQ_CLEAR		  {CANSTMOB &= (~(1<<TXOK));}
#define CAN_RXOK_IRQ_CLEAR		  {CANSTMOB &= (~(1<<RXOK));}



/* Set/Get CAN identifier and mask */

// set extended identifier (only extended supported)
#define CAN_SET_EXT_ID_28_21(identifier) (((*((uint8_t *)(&identifier)+3))<<3)+((*((uint8_t *)(&identifier)+2))>>5))
#define CAN_SET_EXT_ID_20_13(identifier) (((*((uint8_t *)(&identifier)+2))<<3)+((*((uint8_t *)(&identifier)+1))>>5))
#define CAN_SET_EXT_ID_12_5(identifier)  (((*((uint8_t *)(&identifier)+1))<<3)+((* (uint8_t *)(&identifier)   )>>5))
#define CAN_SET_EXT_ID_4_0(identifier)    ((* (uint8_t *)(&identifier)   )<<3)

#define CAN_SET_EXT_ID(identifier) {\
        CANIDT1 = CAN_SET_EXT_ID_28_21 (identifier);\
        CANIDT2 = CAN_SET_EXT_ID_20_13 (identifier);\
        CANIDT3 = CAN_SET_EXT_ID_12_5  (identifier);\
        CANIDT4 = CAN_SET_EXT_ID_4_0   (identifier);\
        CANCDMOB |= (1<<IDE); }
//Get CAN identifier (only extended supported
#define CAN_GET_EXT_ID(identifier){     \
        *((uint8_t *)(&identifier)+3) = CANIDT1>>3;   \
        *((uint8_t *)(&identifier)+2) = (CANIDT2>>3)+(CANIDT1<<5);   \
        *((uint8_t *)(&identifier)+1) = (CANIDT3>>3)+(CANIDT2<<5);   \
        *((uint8_t *)(&identifier))   = (CANIDT4>>3)+(CANIDT3<<5);   \
		}
//Set Remote identifier (not supported yet)
#define CAN_SET_EXT_REMOTE_ID(identifier) {\
        CANIDT1 = CAN_SET_EXT_ID_28_21 (identifier);\
        CANIDT2 = CAN_SET_EXT_ID_20_13 (identifier);\
        CANIDT3 = CAN_SET_EXT_ID_12_5  (identifier);\
        CANIDT4 = CAN_SET_EXT_ID_4_0   (identifier);\
        CANCDMOB |= (1<<IDE); \
        CANIDT4 |= (1<<RTRTAG);}

//set extended mask (only extended supported)
#define CAN_SET_EXT_MSK_28_21(mask) CAN_SET_EXT_ID_28_21(mask)
#define CAN_SET_EXT_MSK_20_13(mask) CAN_SET_EXT_ID_20_13(mask)
#define CAN_SET_EXT_MSK_12_5(mask)  CAN_SET_EXT_ID_12_5(mask)
#define CAN_SET_EXT_MSK_4_0(mask)   CAN_SET_EXT_ID_4_0(mask)

#define CAN_SET_EXT_MASK(mask) {\
		CANIDM1 = CAN_SET_EXT_MSK_28_21 (mask);\
        CANIDM2 = CAN_SET_EXT_MSK_20_13 (mask);\
        CANIDM3 = CAN_SET_EXT_MSK_12_5  (mask);\
        CANIDM4 = CAN_SET_EXT_MSK_4_0   (mask);\
		CANIDM4 = (1<<IDEMSK);}

/* SET MOb modes */

//SET MOb to Enable TX/RX,Disable or Buffer mode (Buffer mode not supported yet
#define DISABLE_CHANNEL        { CANCDMOB &= (~(1<<CONMOB1));CANCDMOB &= (~(1<<CONMOB0));}
#define ENABLE_CHANNEL_TX      { CANCDMOB &= (~(1<<CONMOB1));CANCDMOB |= (1<<CONMOB0);} 
#define ENABLE_CHANNEL_RX      { CANCDMOB |= (1<<CONMOB1);CANCDMOB &= (~(1<<CONMOB0));} 
#define ENABLE_CHANNEL_BUFFER  { CANCDMOB |= (1<<CONMOB1);CANCDMOB |= (1<<CONMOB0);} 

/* Select MOb */

//SET / GET CANPage
#define CAN_SET_CANPAGE(mob) (CANPAGE = (mob << 4))
#define CAN_GET_CANPAGE      (CANPAGE >> 4)

/* CAN helpers */

//get Data length DLC

#define GET_DATA_LENGTH_CODE  (CANCDMOB & 0x0f)


//----- F U N C T I O N S -----


//CAN initailisation
void CAN_Init();

//set CAN interrupts 
//configuration in con_config.h need to be adapted
void CAN_Set_IRQ();

//clear all MOb registers
//called with MOb ID
void CAN_Clear_MOb(uint8_t mob);

//checks whether the mob is free 
//returns TRUE if MOb is unused else FALSE
uint8_t CAN_Get_Free_MOb(uint8_t mob);


//Get MOb with interrupt
//returns number mob with interrupt
uint8_t CAN_Find_MOb_IRQ(void);


//Configure the message object as RX
//mob = 0...MAX_MOB
//CAN_ID_Low = extended CAN_MOb_ID low
//CAN_ID_High = extended CAN_MOb_ID high
//interrupt = ENABLE->interrupt ON; DISABLE->interrupt OFF
//returns TRUE if successful else FALSE
uint8_t CAN_SetRXMOb(uint8_t mob, uint32_t CAN_ID_Low, uint32_t CAN_ID_High, uint8_t interrupt);



//Configure the message object as TX
//mob = 0...MAX_MOB
//interrupt = ENABLE->interrupt ON; DISABLE->interrupt OFF
//returns TRUE if successful else FALSE
uint8_t CAN_SetTXMOb(uint8_t mob, uint8_t interrupt);

//Transmitt CAN Frame
//pointer to meassage object 
//returns TRUE if successful else FALSE
uint8_t CAN_TX_Frame(struct MOb_struct *pt_mob);

//Polls for TXOK
//pointer to meassage object
//if transmission is compleated returns TRUE else FALSE
uint8_t CAN_TX_OK_Poll(struct MOb_struct *pt_mob);

//Polls for RXOK
//pointer to meassage object
//if reseption is compleated returns TRUE else FALSE
uint8_t CAN_RX_OK_Poll(struct MOb_struct *pt_mob);

//Called by interrupt if RX interrupt occured
//number of MOb
//pointer to meassage object
void CAN_Fct_RXOK_IRQ(uint8_t mob,struct MOb_struct *pt_mob);


//Called by interrupt if TX interrupt occured
//number of MOb
void CAN_Fct_TXOK_IRQ(uint8_t mob);

//Called by interrupt if ERROR interrupt occured
//number of MOb
void CAN_Fct_Error_IRQ(uint8_t mob);



#endif // _CAN_H_


